// CONSOLE.C - To be used with Technologic Systems Embedded PCs.
//	It allows the user to switch the console redirection port and baud rate
//
// Revision history
//
// cjd	980512	1.0	Initial version
// cjd	980531	1.1	Set RTS high on the serial port we are switching to,
//			but ONLY if the baud rate is set as well.
// cjd	19990915 1.2	Handle TS/PS returned by PROSOFT PS-4100

#include <stdio.h>
#include <stdlib.h>
#include <dos.h>

int CheckTSBios ();
void SetConsolePort (int);
void SetConsoleSpeed (int, long);
void Exiting ();

#define BOOLEAN unsigned short
#define FALSE	0
#define TRUE	(!FALSE)

BOOLEAN	Quiet = FALSE;                  // flag - if true, don't print anything

const char VER[] = "CONSOLE ver. 1.2, (c) 1998 Technologic Systems Inc.";

// Check whether we are running on a TS EPC
int CheckTSBios ()
{
	union REGS regs;

	regs.x.ax = 0xB000;		// TS BIOS info function
	int86 (0x15, &regs, &regs);     // do it.

	if (regs.h.ah == 0 && regs.x.dx == 0x5453 ||
		regs.h.ah == 0 && regs.x.dx == 0x5053) // 'TS' should be returned in DX

		return (1);             // YES this is TS BIOS
	else
		return (0);		// NO this is NOT TS BIOS
} // CheckTSBios

// Make the BIOS call to change the console port. 0 = Video, 1 = COM1, etc
void SetConsolePort (int port)
{
	union REGS regs;

	regs.h.ah = 0xA1;		// Set console redirection function
	regs.x.bx = port;		// Set which port

	int86 (0x15, &regs, &regs);     // do it.
}


// Set the UART to the baud rate we want
//
// cjd	980531	1.1	Assert RTS on the port too.
//
void SetConsoleSpeed (int Port, long Speed)
{
	unsigned short far *BaseAdr;	// ptr to base register for this serial port
	unsigned short Base,		// base reg. for this serial port
		PortCtrl,		// line control register for the serial port
		PortMcr;		// modem control register for the serial port

	BaseAdr = MK_FP (0x40, (0x00 + 2*(Port - 1)));
	Base = *BaseAdr;
	PortCtrl = Base + 3;		// Port = line control register port #.
	PortMcr = Base + 4;		// Port = modem control register port #.

	switch (Speed) {
		case 50:	Speed = 0x0900; break;
		case 75:	Speed = 0x0600; break;
		case 110:	Speed = 0x0417; break;
		case 134:	Speed = 0x0359; break;
		case 150:	Speed = 0x0300; break;
		case 300:	Speed = 0x0180; break;
		case 600:	Speed = 0x00C0; break;
		case 12:	Speed = 0x0060; break;
		case 1200:	Speed = 0x0060; break;
		case 18:	Speed = 0x0040; break;
		case 1800:	Speed = 0x0040; break;
		case 24:	Speed = 0x0030; break;
		case 2400:	Speed = 0x0030; break;
		case 48:	Speed = 0x0018; break;
		case 4800:	Speed = 0x0018; break;
		case 96:	Speed = 0x000C; break;
		case 9600:	Speed = 0x000C; break;
		case 19:	Speed = 0x0006; break;
		case 19200:	Speed = 0x0006; break;
		case 38:	Speed = 0x0003; break;
		case 38400:	Speed = 0x0003; break;	// 38.4kb/sec.
		case 57:	Speed = 0x0002; break;
		case 57600:	Speed = 0x0002; break;	// 57kb/sec.
		case 115:	Speed = 0x0001; break;	// 115kb/sec.
		case 115200:	Speed = 0x0001; break;	// 115kb/sec.
		default:
			if (!Quiet) printf ("%lu is an invaild baud rate.\n", Speed);
			Exiting ();	// indicate failure.
    }

    //
    // Set DLAB to output the baud rate to the 8250.
    //

    outportb (PortCtrl, inportb (PortCtrl) | 0x80);	// turn on DLAB.

    //
    // Dump the baud rate to the 8250 in two parts.
    //

    outportb (Base, (Speed & 0x00ff));	// output low half.
    outportb (Base + 1, Speed >> 8);	// output high half.

    //
    // Reset DLAB again; we're done with the baud rate.
    //

    outportb (PortCtrl, inportb (PortCtrl) & 0x007f);	// turn off DLAB.

    //
    // Set port to 8 data, 1 stop, no parity
    //

    outportb (PortCtrl, (inp (PortCtrl) & ~0x38) | 0x03);

    //
    // Assert RTS
    //

    outportb (PortMcr, (inp (PortMcr) | 0x02));

} // SetConsoleSpeed

// Display a usage message
void Exiting ()
{
	printf ("%s\n", VER);	// print program version
	printf ("Usage:  CONSOLE /P:port [/S:speed] [/Q]\n");
	printf ("  /P:port     Redirects the console, where port can be:\n");
	printf ("              0     = Standard Console (Video adapter output / serial input)\n");
	printf ("              1 - 4 = COM1 - COM4\n");
	printf ("  /S:speed    Changes the console baud rate, where speed can be\n");
	printf ("              any standard baud rate from 50 to 115200.\n");
	printf ("  /Q          Quiet mode - no output to console.\n\n");
	printf ("If the baud rate is changed, the serial port will also be forced to 8 data\n");
	printf ("bits, 1 stop bit, and no parity.\n");


	exit (1);
} // Exiting


void main(int argc, char *argv[])
{
	int	i,                      // loop control
		port = -1;		// serial port to redirect to
	long	speed = 0;		// baud rate for serial port
	char	*p;                     // argument pointer

	// Check for TS BIOS
	if (!CheckTSBios()) {
		printf ("%s\n", VER);	// print program version
		printf (" This utility is used to change the console port and baud rate\n");
		printf (" for computers with a Technologic Systems BIOS only.\n");

		exit (2);
	}

	// check for bad # of arguments
	if (argc < 2 || argc > 4) {
		Exiting();
	}
	// get the arguments
	for (i = 1; i < argc; i++) {
		p = argv [i];
		strupr (p);
		while (*p) {
			if (*p++ == '/') {
				switch (*p++) {
					case '?':
						Exiting();
						continue;
					case 'H':
						Exiting();
						continue;
					case 'P':
						if (*p++ == ':') {
							port = atoi (p++);
							if (port < 0 || port > 4) {
								Exiting();
							}
						} else
							Exiting();
						continue;
					case 'S':
						if (*p++ == ':') {
							speed = atol (p);
							if (speed < 50 || speed > 115200) {
								Exiting();
							}
						} else
							Exiting();
						continue;
					case 'Q':           // QUIET MODE
						Quiet = TRUE;
						continue;
					default:
						Exiting();
				}
			}
		}
	}

	if (!Quiet) printf ("%s\n", VER);	// print program version

	if (port < 0)
		Exiting ();

	if (port) {
		if (!Quiet) printf ("Redirecting console to COM%u", port);
		if (speed) {
			if (!Quiet) printf (", %lu baud, 8,n,1.\n", speed);
			SetConsoleSpeed (port, speed);
		} else
			if (!Quiet) printf (".\n");
		SetConsolePort (port);
		if (!Quiet) printf ("Console redirected to COM%u", port);
		if (speed)
			if (!Quiet) printf (", %lu baud, 8,n,1.\n", speed);
		else
			if (!Quiet) printf (".\n", speed);
	} else {
		if (!Quiet) printf ("Disabling console redirection.\n");
		SetConsolePort (port);
		if (!Quiet) printf ("Console redirection disabled.\n");
		exit (0);
	}

	exit (0);

} // Main